<?php
/**
* 2007-2020 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  Conexiaweb.es
*  @author conexiaweb.es <info@conexiaweb.es>
*  @copyright  2020  Conexiaweb.es
*  @license    http://conexiaweb.es
*  Great!!!    http://transfertomalaga.es | http://transfertocordoba.es | http://transfertomarbella.com/ | http://transfervalladolid.es/ | http://transfertosevilla.es/ | http://transfertogranada.es/ | http://transfertorremolinos.es/ | http://transferfuengirola.es/ | http://transferbenalmadena.es/ | http://transfertomarbella.com/ | http://transfertonerja.es/ | http://transfertomijas.es/ | http://transfertorredelmar.es/ | http://laborser.es | http://multiserviciosencordoba.es
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class Combinastock extends Module
{
  protected $config_form = false;

  public function __construct()
  {
    $this->name = 'combinastock';
    $this->tab = 'content_management';
    $this->version = '1.0.0';
    $this->author = 'Conexiaweb';
    $this->need_instance = 0;

    $this->bootstrap = true;

    parent::__construct();

    $this->displayName = $this->l('Combinación por defecto con stock PS 1.7');
    $this->description = $this->l('Actualiza la combinación por defecto del producto para que sea siempre una con stock.');

    $this->confirmUninstall = $this->l('Are you sure you want uninstall the module?');

    $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
  }

  public function install()
  {
    return parent::install() &&
      $this->installFixtures() &&
      $this->registerHook('actionUpdateQuantity');
  }

  public function uninstall()
  {
    return $this->uninstallFixtures() && parent::uninstall();
  }
    
  protected function installFixtures()
  {
    return  Configuration::updateValue('COMBINASTOCK_ACTIVE', 0) &&
            Configuration::updateValue('COMBINASTOCK_COMB_SELECT_MODE', 'price_lower');
  }
  
  protected function uninstallFixtures()
  {
    return  Configuration::deleteByName('COMBINASTOCK_ACTIVE') &&
            Configuration::deleteByName('COMBINASTOCK_COMB_SELECT_MODE');
  }
  
  public function hookActionUpdateQuantity($params)
  {
    if (  Configuration::get('COMBINASTOCK_ACTIVE') &&
          Configuration::get('PS_STOCK_MANAGEMENT') &&
          !Configuration::get('PS_ORDER_OUT_OF_STOCK') &&
          $params['quantity']==0 && $params['id_product_attribute']!=0
    ) {
      /* Check if stock modified combination is default */
      $defaultAttributeCombination = (int) Product::getDefaultAttribute($params['id_product']);
      if ($params['id_product_attribute'] == $defaultAttributeCombination) {
        $this->selectNewCombinationForProduct($params['id_product']);
      }
    }
  }
  
  protected function selectNewCombinationForProduct($idProduct)
  {
    $product = new Product($idProduct);
    $combinations = $product->getAttributeCombinations($this->context->language->id, false);
    if (!empty($combinations)) {
      $lowerStocked = $highterStocked = array('id'=>0, 'stock'=>null);
      $lowerPriced = $highterPriced = array('id'=>0, 'price'=>null);
      $newCombinationAvailable = false;
      foreach ($combinations as $comb) {
        $price = Product::getPriceStatic((int)$product->id, true, $comb['id_product_attribute'], 2, null, false, true);
        $stock = Product::getQuantity($product->id, $comb['id_product_attribute']);
        if ($stock > 0) {
          if (!is_numeric($lowerPriced['price']) || ($lowerPriced['price'] > $price)) {
            $lowerPriced = array ('id'=>$comb['id_product_attribute'], 'price'=>$price);
          }
          if (!is_numeric($highterPriced['price']) || ($highterPriced['price'] < $price)) {
            $highterPriced = array ('id'=>$comb['id_product_attribute'], 'price'=>$price);
          }
          if (!is_numeric($lowerStocked['stock']) || ($lowerStocked['stock'] > $stock)) {
            $lowerStocked = array ('id'=>$comb['id_product_attribute'], 'stock'=>$stock);
          }
          if (!is_numeric($highterStocked['stock']) || ($highterStocked['stock'] < $stock)) {
            $highterStocked = array ('id'=>$comb['id_product_attribute'], 'stock'=>$stock);
          }
          $newCombinationAvailable = true;
        }
      }
      /* Set combination for product */
      if ($newCombinationAvailable) {
        $product->deleteDefaultAttributes();
        switch (Configuration::get('COMBINASTOCK_COMB_SELECT_MODE')) {
          case 'price_lower':
            $product->setDefaultAttribute($lowerPriced['id']);
            break;
          case 'price_highter':
            $product->setDefaultAttribute($highterPriced['id']);
            break;
          case 'stock_lower':
            $product->setDefaultAttribute($lowerStocked['id']);
            break;
          case 'stock_highter':
            $product->setDefaultAttribute($highterStocked['id']);
            break;
          default:
            $product->setDefaultAttribute($lowerPriced['id']);
            break;
        }
      }
    }
  }

  /**
   * Load the configuration form
   */
  public function getContent()
  {

    if (((bool)Tools::isSubmit('submitAutoupdatecombinationModule')) == true) {
      $this->postProcess();
    }

    $this->context->smarty->assign('module_dir', $this->_path);
    $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');
    return $output.$this->renderForm();
  }


  protected function renderForm()
  {
    $helper = new HelperForm();

    $helper->show_toolbar = false;
    $helper->table = $this->table;
    $helper->module = $this;
    $helper->default_form_language = $this->context->language->id;
    $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

    $helper->identifier = $this->identifier;
    $helper->submit_action = 'submitAutoupdatecombinationModule';
    $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
        .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
    $helper->token = Tools::getAdminTokenLite('AdminModules');

    $helper->tpl_vars = array(
        'fields_value' => $this->getConfigFormValues(),
        'languages' => $this->context->controller->getLanguages(),
        'id_language' => $this->context->language->id,
    );

    return $helper->generateForm(array($this->getConfigForm()));
  }


  protected function getConfigForm()
  {
    $combinationCriteria = array(
      array('id'=>'stock_highter', 'name'=>$this->l('Highter stock')),
      array('id'=>'stock_lower', 'name'=>$this->l('Lower stock')),
      array('id'=>'price_highter', 'name'=>$this->l('Price highter')),
      array('id'=>'price_lower', 'name'=>$this->l('Price lower'))
    );
    
    return array(
      'form' => array(
        'legend' => array(
        'title' => $this->l('Settings'),
        'icon' => 'icon-cogs',
        ),
        'input' => array(
          array(
            'type' => 'switch',
            'label' => $this->l('Module active'),
            'name' => 'COMBINASTOCK_ACTIVE',
            'is_bool' => true,
            'desc' => $this->l('Active or deactive module functions'),
            'values' => array(
              array(
                'id' => 'active_on',
                'value' => true,
                'label' => $this->l('Enabled')
              ),
              array(
                'id' => 'active_off',
                'value' => false,
                'label' => $this->l('Disabled')
              )
            ),
          ),
          array(
            'type' => 'select',
            'label' => $this->l('Default combination criteria'),
            'name' => 'COMBINASTOCK_COMB_SELECT_MODE',
            'options' => array(
              'query' => $combinationCriteria,
              'id' => 'id',
              'name' => 'name'
            ),
            'desc' => $this->l('Select criteria to choose default combination when main default combination\'s stock reaches 0.'),
            'class' => 'fixed-width-xxl'
          ),
        ),
        'submit' => array(
          'title' => $this->l('Save'),
        ),
      ),
    );
  }


  protected function getConfigFormValues()
  {
    return array(
      'COMBINASTOCK_ACTIVE' => (bool)Configuration::get('COMBINASTOCK_ACTIVE', false),
      'COMBINASTOCK_COMB_SELECT_MODE' => Configuration::get('COMBINASTOCK_COMB_SELECT_MODE', 'price_lower'),
    );
  }

  protected function postProcess()
  {
    if (Tools::isSubmit('submitAutoupdatecombinationModule')) {
      $form_values = $this->getConfigFormValues();
      foreach (array_keys($form_values) as $key) {
        Configuration::updateValue($key, Tools::getValue($key));
      }
      return $this->displayConfirmation($this->l('The settings have been updated'));
    }
  }
}
